# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2022 CERN.
# Copyright (C) 2023 KTH Royal Institute of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Page module config."""

from invenio_i18n import lazy_gettext as _

PAGES_DEFAULT_TEMPLATE = "invenio_pages/default.html"
"""Default template to render."""

PAGES_TEMPLATES = [
    ("invenio_pages/default.html", "Default"),
    ("invenio_pages/dynamic.html", "Default dynamic"),
]
"""List of available templates for pages."""

PAGES_WHITELIST_CONFIG_KEYS = [
    "THEME_SITENAME",
]
"""List of configuration variables accessible in the dynamic pages."""

PAGES_SEARCH = {
    "facets": [],
    "sort": ["url", "title", "created", "updated"],
}
"""Page search configuration (i.e list of pages)."""

PAGES_SORT_OPTIONS = {
    "url": dict(
        title=_("URL"),
        fields=["url"],
    ),
    "title": dict(
        title=_("Title"),
        fields=["title"],
    ),
    "created": dict(
        title=_("Created"),
        fields=["created"],
    ),
    "updated": dict(
        title=_("Updated"),
        fields=["updated"],
    ),
}
"""Definitions of available record sort options."""

PAGES_FACETS = {}
"""Available facets defined for this module."""


PAGES_ALLOWED_EXTRA_HTML_TAGS = ["img", "button"]
"""Extend allowed HTML tags list for static pages content."""

PAGES_ALLOWED_EXTRA_HTML_ATTRS = {
    "img": ["src", "alt", "title", "width", "height", "loading"],
    "button": ["type", "name", "value", "disabled", "onclick"],
}
"""Extend allowed HTML attrs list  for static pages content."""



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2022 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Create pages table."""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "145402e8523a"
down_revision = "98e0a418340d"
branch_labels = ()
depends_on = None


def upgrade():
    """Upgrade database."""
    op.create_table(
        "pages_page",
        sa.Column("created", sa.DateTime(), nullable=False),
        sa.Column("updated", sa.DateTime(), nullable=False),
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("url", sa.String(length=100), nullable=False),
        sa.Column("title", sa.String(length=200), nullable=False),
        sa.Column("content", sa.Text(), nullable=False),
        sa.Column("description", sa.String(length=200), nullable=False),
        sa.Column("template_name", sa.String(length=70), nullable=False),
        sa.PrimaryKeyConstraint("id", name=op.f("pk_pages_page")),
        sa.UniqueConstraint("url", name=op.f("uq_pages_page_url")),
    )

    op.create_table(
        "pages_page_version",
        sa.Column("created", sa.DateTime(), autoincrement=False, nullable=True),
        sa.Column("updated", sa.DateTime(), autoincrement=False, nullable=True),
        sa.Column("id", sa.Integer(), autoincrement=False, nullable=False),
        sa.Column("url", sa.String(length=100), autoincrement=False, nullable=True),
        sa.Column("title", sa.String(length=200), autoincrement=False, nullable=True),
        sa.Column("content", sa.Text(), autoincrement=False, nullable=True),
        sa.Column(
            "description", sa.String(length=200), autoincrement=False, nullable=True
        ),
        sa.Column(
            "template_name", sa.String(length=70), autoincrement=False, nullable=True
        ),
        sa.Column(
            "transaction_id", sa.BigInteger(), autoincrement=False, nullable=False
        ),
        sa.Column("end_transaction_id", sa.BigInteger(), nullable=True),
        sa.Column("operation_type", sa.SmallInteger(), nullable=False),
        sa.PrimaryKeyConstraint(
            "id", "transaction_id", name=op.f("pk_pages_page_version")
        ),
    )

    op.create_index(
        op.f("ix_pages_page_version_end_transaction_id"),
        "pages_page_version",
        ["end_transaction_id"],
        unique=False,
    )
    op.create_index(
        op.f("ix_pages_page_version_operation_type"),
        "pages_page_version",
        ["operation_type"],
        unique=False,
    )
    op.create_index(
        op.f("ix_pages_page_version_transaction_id"),
        "pages_page_version",
        ["transaction_id"],
        unique=False,
    )

    op.create_table(
        "pages_pagelist_version",
        sa.Column("id", sa.Integer(), autoincrement=False, nullable=False),
        sa.Column("list_id", sa.Integer(), autoincrement=False, nullable=True),
        sa.Column("page_id", sa.Integer(), autoincrement=False, nullable=True),
        sa.Column("order", sa.Integer(), autoincrement=False, nullable=True),
        sa.Column(
            "transaction_id", sa.BigInteger(), autoincrement=False, nullable=False
        ),
        sa.Column("end_transaction_id", sa.BigInteger(), nullable=True),
        sa.Column("operation_type", sa.SmallInteger(), nullable=False),
        sa.PrimaryKeyConstraint(
            "id", "transaction_id", name=op.f("pk_pages_pagelist_version")
        ),
    )
    op.create_index(
        op.f("ix_pages_pagelist_version_end_transaction_id"),
        "pages_pagelist_version",
        ["end_transaction_id"],
        unique=False,
    )
    op.create_index(
        op.f("ix_pages_pagelist_version_operation_type"),
        "pages_pagelist_version",
        ["operation_type"],
        unique=False,
    )
    op.create_index(
        op.f("ix_pages_pagelist_version_transaction_id"),
        "pages_pagelist_version",
        ["transaction_id"],
        unique=False,
    )
    op.create_table(
        "pages_pagelist",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("list_id", sa.Integer(), nullable=False),
        sa.Column("page_id", sa.Integer(), nullable=False),
        sa.Column("order", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["list_id"],
            ["pages_page.id"],
            name=op.f("fk_pages_pagelist_list_id_pages_page"),
        ),
        sa.ForeignKeyConstraint(
            ["page_id"],
            ["pages_page.id"],
            name=op.f("fk_pages_pagelist_page_id_pages_page"),
        ),
        sa.PrimaryKeyConstraint("id", name=op.f("pk_pages_pagelist")),
    )


def downgrade():
    """Downgrade database."""
    op.drop_table("pages_pagelist")
    op.drop_index(
        op.f("ix_pages_pagelist_version_transaction_id"),
        table_name="pages_pagelist_version",
    )
    op.drop_index(
        op.f("ix_pages_pagelist_version_operation_type"),
        table_name="pages_pagelist_version",
    )
    op.drop_index(
        op.f("ix_pages_pagelist_version_end_transaction_id"),
        table_name="pages_pagelist_version",
    )
    op.drop_table("pages_pagelist_version")
    op.drop_index(
        op.f("ix_pages_page_version_transaction_id"), table_name="pages_page_version"
    )
    op.drop_index(
        op.f("ix_pages_page_version_operation_type"), table_name="pages_page_version"
    )
    op.drop_index(
        op.f("ix_pages_page_version_end_transaction_id"),
        table_name="pages_page_version",
    )
    op.drop_table("pages_page_version")
    op.drop_table("pages_page")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
# Copyright (C) 2025 Northwestern University.
#
# Invenio-Pages is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Page Service API."""

from .config import PageServiceConfig
from .results import PageItem
from .service import PageService

__all__ = (
    "PageService",
    "PageServiceConfig",
    "PageList",
    "PageItem",
)



#
# This file is part of Invenio.
# Copyright (C) 2016-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Add has_custom_view to pages."""

import sqlalchemy as sa
import sqlalchemy_utils
from alembic import op
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "b0f93ca4a147"
down_revision = "145402e8523a"
branch_labels = ()
depends_on = "98e0a418340d"


def upgrade():
    """Upgrade database."""
    # Add the new column
    op.add_column(
        "pages_page",
        sa.Column(
            "has_custom_view",
            sa.Boolean(),
            nullable=False,
            server_default=sa.sql.expression.literal(False),
            default=False,
        ),
    )
    op.add_column(
        "pages_page_version",
        sa.Column(
            "has_custom_view",
            sa.Boolean(),
            nullable=True,
            server_default=sa.sql.expression.literal(False),
            default=False,
        ),
    )

    # ### end Alembic commands ###


def downgrade():
    """Downgrade database."""
    # Drop the column
    op.drop_column("pages_page", "has_custom_view")
    op.drop_column("pages_page_version", "has_custom_view")
    # ### end Alembic commands ###



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2025 CERN.
#
# Invenio-Pages is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Page Service API."""

from datetime import datetime

import sqlalchemy as sa
from invenio_records_resources.services import RecordService
from invenio_records_resources.services.base import LinksTemplate
from invenio_records_resources.services.base.utils import map_search_params
from invenio_records_resources.services.uow import unit_of_work
from sqlalchemy import func

from ..records.models import PageModel as Page


class PageService(RecordService):
    """Page Service."""

    @unit_of_work()
    def create(self, identity, data, uow=None):
        """Create a page."""
        self.require_permission(identity, "create")
        data, errors = self.schema.load(
            data,
            context={"identity": identity},
            raise_errors=False,
        )
        page = self.record_cls.create(data)

        response = self.result_item(
            self, identity, page, links_tpl=self.links_item_tpl, errors=errors
        )
        return response

    def read_by_url(self, identity, url, lang):
        """Retrieve a page by url."""
        self.require_permission(identity, "read")
        page = self.record_cls.get_by_url(url, lang=lang)
        response = self.result_item(self, identity, page, links_tpl=self.links_item_tpl)
        return response

    def read(self, identity, id):
        """Retrieve a page."""
        self.require_permission(identity, "read")
        page = self.record_cls.get(id)
        response = self.result_item(self, identity, page, links_tpl=self.links_item_tpl)
        return response

    def search(self, identity, params={}):
        """Search for pages matching the querystring."""
        self.require_permission(identity, "search")

        filters = []
        search_params = map_search_params(self.config.search, params)
        params.update(size=search_params["size"], page=search_params["page"])
        query = search_params["q"]
        if query:
            filters.append(self._query_filter(query))

        pages = self.record_cls.search(search_params, filters)
        return self.result_list(
            self,
            identity,
            pages,
            params=params,
            links_tpl=LinksTemplate(self.config.links_search, context={"args": params}),
            links_item_tpl=self.links_item_tpl,
        )

    @unit_of_work()
    def update(self, identity, data, id, uow=None):
        """Update a page."""
        self.require_permission(identity, "update")
        data, errors = self.schema.load(
            data,
            context={"identity": identity},
            raise_errors=False,
        )
        self.record_cls.update(data, id)
        page = self.record_cls.get(id)
        response = self.result_item(
            self, identity, page, links_tpl=self.links_item_tpl, errors=errors
        )
        return response

    @unit_of_work()
    def delete(self, identity, id, uow=None):
        """Delete a page."""
        self.require_permission(identity, "delete")
        page = self.record_cls.get(id)
        self.record_cls.delete(page)
        response = self.result_item(self, identity, page, links_tpl=self.links_item_tpl)
        return response

    @unit_of_work()
    def delete_all(self, identity, uow=None):
        """Delete all pages."""
        self.require_permission(identity, "delete")
        self.record_cls.delete_all()

    def _query_filter(self, query):
        """Build an SQLAlchemy filter for the search query."""
        clauses = [
            Page.url.ilike(f"%{query}%"),
            Page.title.ilike(f"%{query}%"),
            Page.content.ilike(f"%{query}%"),
            Page.description.ilike(f"%{query}%"),
            Page.template_name.ilike(f"%{query}%"),
        ]
        datetime_value = self._parse_datetime(query)
        if datetime_value is not None:
            clauses.extend(
                [
                    func.date(Page.created) == datetime_value,
                    func.date(Page.updated) == datetime_value,
                ]
            )
        return sa.or_(*clauses)

    def _parse_datetime(self, value):
        """Parse a datetime string in the format YYYY-MM-DD."""
        try:
            datetime_value = datetime.strptime(value, "%Y-%m-%d")
        except ValueError:
            return None
        return datetime_value



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2025 CERN.
# Copyright (C) 2025      University of Münster.
# Copyright (C) 2025 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Pages module models."""

from flask import current_app
from invenio_db import db
from sqlalchemy.exc import IntegrityError, NoResultFound
from sqlalchemy.orm import validates
from sqlalchemy.sql import text
from sqlalchemy_utils.models import Timestamp

from .errors import PageNotCreatedError, PageNotFoundError


class PageModel(db.Model, Timestamp):
    """Represents a page."""

    __versioned__ = {}

    __tablename__ = "pages_page"

    id = db.Column(db.Integer, nullable=False, primary_key=True, autoincrement=True)
    """Page identifier."""

    url = db.Column(db.String(100), nullable=False)
    """Page url."""

    title = db.Column(db.String(200), nullable=False, default="")
    """Page title."""

    lang = db.Column(db.CHAR(2), nullable=False, server_default="en")
    """Page language. Default is 'en'"""

    content = db.Column(db.Text(), nullable=False, default="")
    """Page content. Default is pages/templates/default.html"""

    description = db.Column(db.String(200), nullable=False, default="")
    """Page description."""

    template_name = db.Column(db.String(70), nullable=False)
    """Page template name."""

    __table_args__ = (
        db.UniqueConstraint("url", "lang", name="uq_pages_page_url_lang"),
    )

    @classmethod
    def create(cls, data):
        """Create a new page."""
        try:
            with db.session.begin_nested():
                obj = cls(
                    url=data["url"],
                    title=data.get("title", ""),
                    content=data.get("content", ""),
                    lang=data.get("lang", "en"),
                    description=data.get("description", ""),
                    template_name=data["template_name"],
                )
                db.session.add(obj)

            return obj
        except IntegrityError:
            raise PageNotCreatedError(data["url"])

    @classmethod
    def get_by_url(cls, url, lang="en"):
        """Get a page by URL.

        :param url: The page URL.
        :returns: A :class:`invenio_pages.records.models.PageModel` instance.
        """
        try:
            return cls.query.filter_by(url=url, lang=lang).one()
        except NoResultFound:
            raise PageNotFoundError(url)

    @classmethod
    def get(cls, id):
        """Get a page by ID.

        :param id: The page ID.
        :returns: A :class:`invenio_pages.records.models.PageModel` instance.
        """
        try:
            return cls.query.filter_by(id=id).one()
        except NoResultFound:
            raise PageNotFoundError(id)

    @classmethod
    def search(cls, search_params, filters):
        """Get pages according to param filters.

        :param search_params: The maximum resources to retreive.
        :param filters: The search filters.
        :returns: A list of the :class:`invenio_pages.records.models.PageModel` instance.
        """
        pages = (
            cls.query.filter(*filters)
            .order_by(
                search_params["sort_direction"](text(",".join(search_params["sort"])))
            )
            .paginate(
                page=search_params["page"],
                per_page=search_params["size"],
                error_out=False,
            )
        )
        return pages

    @classmethod
    def update(cls, data, id):
        """Update an existing page."""
        with db.session.begin_nested():
            cls.query.filter_by(id=id).update(data)

    @classmethod
    def delete(cls, page):
        """Delete page by its id."""
        with db.session.begin_nested():
            db.session.delete(page)

    @classmethod
    def delete_all(cls):
        """Delete all pages."""
        cls.query.delete()

    @validates("template_name")
    def validate_template_name(self, key, value):
        """Validate template name.

        :param key: The template path.
        :param value: The template name.
        :raises ValueError: If template name is wrong.
        """
        if value not in dict(current_app.config["PAGES_TEMPLATES"]):
            raise ValueError('Template "{0}" does not exist.'.format(value))
        return value

    def __repr__(self):
        """Page representation.

        Used on Page admin view in inline model.
        :returns: unambiguous page representation.
        """
        return f"URL: {self.url}, title: {self.title}, language: {self.lang}"


class PageList(db.Model):
    """Represent association between page and list."""

    __versioned__ = {}

    __tablename__ = "pages_pagelist"

    id = db.Column(db.Integer, nullable=False, primary_key=True, autoincrement=True)
    """PageList identifier."""

    list_id = db.Column(db.Integer, db.ForeignKey(PageModel.id), nullable=False)
    """Id of a list."""

    page_id = db.Column(db.Integer, db.ForeignKey(PageModel.id), nullable=False)
    """Id of a page."""

    order = db.Column(db.Integer, nullable=False)

    list = db.relationship(
        PageModel,
        backref=db.backref("pages", cascade="all, delete-orphan"),
        foreign_keys=[list_id],
    )
    """Relation to the list."""

    page = db.relationship(
        PageModel,
        backref=db.backref("lists", cascade="all, delete-orphan"),
        foreign_keys=[page_id],
    )
    """Relation to the page."""



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2025 CERN.
# Copyright (C) 2023-2024 Graz University of Technology.
# Copyright (C) 2025      University of Münster.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Views for Pages module."""

from flask import Blueprint, abort, current_app, g, render_template, request
from invenio_db import db
from invenio_i18n import get_locale
from sqlalchemy.orm.exc import NoResultFound
from werkzeug.exceptions import NotFound

from .proxies import current_pages_service
from .records.models import PageModel as Page

blueprint = Blueprint(
    "invenio_pages", __name__, url_prefix="/", template_folder="templates"
)


@blueprint.app_template_filter("render_string")
def render_string(source):
    """Render a string in sandboxed environment.

    :param source: A string containing the page source.
    :returns: The rendered template.
    """
    return current_app.extensions["invenio-pages"].render_template(source)


def handle_not_found(exception, **extra):
    """Custom blueprint exception handler."""
    assert isinstance(exception, NotFound)

    page = Page.query.filter(
        db.or_(Page.url == request.path, Page.url == request.path + "/")
    ).first()
    if page:
        add_url_rule(page.url)
        return render_template(
            [page.template_name, current_app.config["PAGES_DEFAULT_TEMPLATE"]],
            page=page,
        )
    elif "wrapped" in extra:
        return extra["wrapped"](exception)
    else:
        return exception


def add_url_rule(url, app=None):
    """Register URL rule to application URL map."""
    rule = current_app.url_rule_class(
        url,
        endpoint="invenio_pages.view",
        methods=["GET", "HEAD", "OPTIONS"],
        strict_slashes=True,
        merge_slashes=True,
    )
    current_app.url_map.add(rule)


def create_pages_api_bp(app):
    """Create the pages resource api blueprint."""
    ext = app.extensions["invenio-pages"]
    return ext.pages_resource.as_blueprint()


def render_page(path, **template_ctx):
    """Internal interface to the page view.

    :param path: Page path.
    :param template_ctx: Passed to the rendered template.
    :returns: The rendered template.
    """
    try:
        lang = get_locale()
        page = current_pages_service.read_by_url(
            g.identity, request.path, lang.language
        ).to_dict()
    except NoResultFound:
        abort(404)
    return render_template(
        [page["template_name"], current_app.config["PAGES_DEFAULT_TEMPLATE"]],
        page=page,
        **template_ctx,
    )


def create_page_view(page_path):
    """Create a page view.

    .. code-block:: python

        app = Flask(__name__)
        app.add_url_rule("/about", view_func=create_page_view("/about"))

    """

    def _view():
        """Public interface to the page view.

        Models: `pages.pages`.
        Templates: Uses the template defined by the ``template_name`` field
        or ``pages/default.html`` if template_name is not defined.
        Context: page `pages.pages` object.
        """
        return render_page(page_path)

    return _view



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2024 CERN.
# Copyright (C) 2023-2024 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Static pages module for Invenio."""

import sqlalchemy
from flask import request, url_for
from invenio_db import db
from jinja2.sandbox import SandboxedEnvironment
from werkzeug.exceptions import NotFound

from . import config
from .records.models import PageModel as Page
from .resources import PageResource, PageResourceConfig
from .services import PageService, PageServiceConfig
from .views import add_url_rule, handle_not_found, render_page


class InvenioPages(object):
    """Invenio-Pages extension."""

    def __init__(self, app=None):
        """Extension initialization.

        :param app: The Flask application. (Default: ``None``)
        """
        if app:
            self.init_app(app)

    @staticmethod
    def wrap_errorhandler(app):
        """Wrap error handler.

        :param app: The Flask application.
        """
        try:
            existing_handler = app.error_handler_spec[None][404][NotFound]
        except (KeyError, TypeError):
            existing_handler = None

        if existing_handler:
            app.error_handler_spec[None][404][NotFound] = (
                lambda error: handle_not_found(error, wrapped=existing_handler)
            )
        else:
            app.error_handler_spec.setdefault(None, {}).setdefault(404, {})
            app.error_handler_spec[None][404][NotFound] = handle_not_found

    @property
    def jinja_env(self):
        """Create a sandboxed Jinja environment."""
        if self._jinja_env is None:
            self._jinja_env = SandboxedEnvironment(
                autoescape=True,
            )
            self._jinja_env.globals["url_for"] = url_for
            # Load whitelisted configuration variables.
            for var in self.app.config["PAGES_WHITELIST_CONFIG_KEYS"]:
                self._jinja_env.globals[var] = self.app.config.get(var)
        return self._jinja_env

    def render_template(self, source, **kwargs_context):
        r"""Render a template string using sandboxed environment.

        :param source: A string containing the page source.
        :param \*\*kwargs_context: The context associated with the page.
        :returns: The rendered template.
        """
        return self.jinja_env.from_string(source).render(kwargs_context)

    def init_app(self, app):
        """Flask application initialization.

        :param app: The Flask application.
        :returns: The :class:`invenio_pages.ext.InvenioPages` instance
            initialized.
        """
        self.init_config(app)
        self.init_services(app)
        self.init_resources(app)
        self.wrap_errorhandler(app)
        app.extensions["invenio-pages"] = self

        return app.extensions["invenio-pages"]

    def init_config(self, app):
        """Initialize configuration.

        :param app: The Flask application.
        """
        app.config.setdefault(
            "PAGES_BASE_TEMPLATE",
            app.config.get("BASE_TEMPLATE", "invenio_pages/base.html"),
        )

        for k in dir(config):
            if k.startswith("PAGES_"):
                app.config.setdefault(k, getattr(config, k))

    def init_services(self, app):
        """Initialize the services for pages."""
        self.pages_service = PageService(config=PageServiceConfig)

    def init_resources(self, app):
        """Initialize the resources for pages."""
        self.pages_resource = PageResource(
            service=self.pages_service,
            config=PageResourceConfig,
        )


class InvenioPagesREST(InvenioPages):
    """Invenio App ILS REST API app."""



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2024 CERN.
# Copyright (C) 2025 Graz University of Technology.
#
# Invenio-Pages is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Page Resource Configuration."""
import marshmallow as ma
from invenio_records_resources.resources import (
    RecordResourceConfig,
    SearchRequestArgsSchema,
)


class PageRequestSearchArgs(SearchRequestArgsSchema):
    """Page request search args."""

    sort_direction = ma.fields.Str()


class PageResourceConfig(RecordResourceConfig):
    """Page resource config."""

    # Blueprint configuration
    blueprint_name = "pages"
    url_prefix = "/pages"
    routes = {
        "item": "/<id>",
        "list": "",
    }

    request_search_args = PageRequestSearchArgs
    request_view_args = {
        "id": ma.fields.Integer(),
    }

    response_handlers = {
        "application/vnd.inveniordm.v1+json": RecordResourceConfig.response_handlers[
            "application/json"
        ],
        **RecordResourceConfig.response_handlers,
    }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
# Copyright (C) 2025 Northwestern University.
#
# Invenio-Pages is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Page Service Configuration."""


from invenio_i18n import gettext as _
from invenio_records_resources.services import (
    EndpointLink,
    RecordServiceConfig,
    pagination_endpoint_links,
)
from sqlalchemy import asc, desc

from ..records.models import PageModel
from .permissions import PagesPermissionPolicy
from .results import PageItem, PageList
from .schemas import PageSchema


class SearchOptions:
    """Search options."""

    sort_direction_default = "asc"
    sort_direction_options = {
        "asc": dict(
            title=_("Ascending"),
            fn=asc,
        ),
        "desc": dict(
            title=_("Descending"),
            fn=desc,
        ),
    }

    sort_default = "created"
    sort_options = {
        "id": dict(
            title=_("Id"),
            fields=["id"],
        ),
        "url": dict(
            title=_("Url"),
            fields=["url"],
        ),
        "title": dict(
            title=_("Title"),
            fields=["title"],
        ),
        "content": dict(
            title=_("Content"),
            fields=["content"],
        ),
        "template_name": dict(
            title=_("Template Name"),
            fields=["template_name"],
        ),
        "description": dict(
            title=_("Description"),
            fields=["description"],
        ),
        "created": dict(
            title=_("Created"),
            fields=["created"],
        ),
        "updated": dict(
            title=_("Updated"),
            fields=["updated"],
        ),
    }
    pagination_options = {
        "default_results_per_page": 25,
    }


class PageServiceConfig(RecordServiceConfig):
    """Service factory configuration."""

    permission_policy_cls = PagesPermissionPolicy
    schema = PageSchema
    search = SearchOptions
    result_item_cls = PageItem
    result_list_cls = PageList
    record_cls = PageModel
    links_item = {
        "self": EndpointLink(
            "pages.read",
            params=["id"],
            vars=lambda obj, vars: vars.update(id=obj.id),
        ),
    }
    links_search = pagination_endpoint_links("pages.search")



#
# This file is part of Invenio.
# Copyright (C) 2025 University of Münster.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Add language and new uq."""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "1737449793"
down_revision = "9fae3c5404d9"
branch_labels = ()
depends_on = None


def upgrade():
    """Upgrade database."""
    op.drop_constraint(op.f("uq_pages_page_url"), "pages_page", type_="unique")
    op.add_column(
        "pages_page",
        sa.Column(
            "lang",
            sa.CHAR(length=2),
            server_default="en",
            nullable=False,
        ),
    )
    op.create_unique_constraint("uq_pages_page_url_lang", "pages_page", ["url", "lang"])
    op.add_column(
        "pages_page_version",
        sa.Column(
            "lang",
            sa.CHAR(length=2),
            server_default="en",
            nullable=True,
        ),
    )


def downgrade():
    """Downgrade database."""
    op.drop_column("pages_page_version", "lang")
    op.drop_constraint("uq_pages_page_url_lang", "pages_page", type_="unique")
    op.drop_column("pages_page", "lang")
    op.create_unique_constraint(op.f("uq_pages_page_url"), "pages_page", ["url"])



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
#
# Invenio-Pages is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Errors."""

from invenio_i18n import gettext as _


class PageNotFoundError(Exception):
    """Page not found exception."""

    def __init__(self, identifier):
        """Initialise error."""
        super().__init__(
            _(
                "The page identified by {identifier} cannot be found.".format(
                    identifier=identifier
                )
            )
        )


class PageNotCreatedError(Exception):
    """Page not created exception."""

    def __init__(self, url):
        """Initialise error."""
        super().__init__(
            _(
                "The page with url {url} couldn't be created, likely due to a page with the same URL already existing.".format(
                    url=url
                )
            )
        )
